@@ -4,23 +4,12 @@ import argparse
 import inspect
 import os
 from abc import ABC
-from typing import (
-    TYPE_CHECKING,
-    Any,
-    AsyncIterator,
-    Callable,
-    Dict,
-    Iterator,
-    Optional,
-    Tuple,
-    Union,
-)
+from typing import TYPE_CHECKING, AsyncIterator, Callable, Iterator, Optional, Union
 
 from jina.excepts import BadClientInput
-from jina.helper import ArgNamespace, T, typename
+from jina.helper import T, parse_client, typename
 from jina.logging.logger import JinaLogger
 from jina.logging.predefined import default_logger
-from jina.parsers import set_client_cli_parser
 
 if TYPE_CHECKING:
     from jina.clients.request import GeneratorSourceType
@@ -45,10 +34,7 @@ class BaseClient(ABC):
         if args and isinstance(args, argparse.Namespace):
             self.args = args
         else:
-            self._parse_kwargs(kwargs)
-            self.args = ArgNamespace.kwargs2namespace(
-                kwargs, set_client_cli_parser(), warn_unknown=True
-            )
+            self.args = parse_client(kwargs)
 
         self.logger = JinaLogger(self.__class__.__name__, **vars(self.args))
 
@@ -61,39 +47,6 @@ class BaseClient(ABC):
             os.unsetenv('https_proxy')
         self._inputs = None
 
-    def _parse_kwargs(self, kwargs: Dict[str, Any]):
-
-        if 'host' in kwargs.keys():
-            return_scheme = dict()
-            (
-                kwargs['host'],
-                return_scheme['port'],
-                return_scheme['protocol'],
-                return_scheme['tls'],
-            ) = self._parse_host_scheme(kwargs['host'])
-
-            for key, value in return_scheme.items():
-                if value:
-                    if key in kwargs:
-                        raise ValueError(
-                            f"You can't have two definitions of {key}: you have one in the host scheme and one in the keyword argument"
-                        )
-                    elif value:
-                        kwargs[key] = value
-
-    def _parse_host_scheme(self, host: str) -> Tuple[str, str, str, bool]:
-        scheme, _hostname, port = _parse_url(host)
-
-        tls = None
-        if scheme in ('grpcs', 'https', 'wss'):
-            scheme = scheme[:-1]
-            tls = True
-
-        if scheme == 'ws':
-            scheme = 'websocket'
-
-        return _hostname, port, scheme, tls
-
     @staticmethod
     def check_input(inputs: Optional['InputType'] = None, **kwargs) -> None:
         """Validate the inputs and print the first request if success.
@@ -145,9 +98,16 @@ class BaseClient(ABC):
         _kwargs.update(kwargs)
 
         if hasattr(self._inputs, '__len__'):
-            self._inputs_length = max(1, len(self._inputs) / _kwargs['request_size'])
+            total_docs = len(self._inputs)
+        elif 'total_docs' in _kwargs:
+            total_docs = _kwargs['total_docs']
         else:
-            self._inputs_length = None
+            total_docs = None
+
+        self._inputs_length = None
+
+        if total_docs:
+            self._inputs_length = max(1, total_docs / _kwargs['request_size'])
 
         if inspect.isasyncgen(self.inputs):
             from jina.clients.request.asyncio import request_generator
@@ -192,6 +152,14 @@ class BaseClient(ABC):
     ):
         ...
 
+    @abc.abstractmethod
+    def _dry_run(self, **kwargs) -> bool:
+        """Sends a dry run to the Flow to validate if the Flow is ready to receive requests
+
+        :param kwargs: potential kwargs received passed from the public interface
+        """
+        ...
+
     @property
     def client(self: T) -> T:
         """Return the client object itself
@@ -199,17 +167,3 @@ class BaseClient(ABC):
         :return: the Client object
         """
         return self
-
-
-def _parse_url(host):
-    if '://' in host:
-        scheme, host = host.split('://')
-    else:
-        scheme = None
-
-    if ':' in host:
-        host, port = host.split(':')
-    else:
-        port = None
-
-    return scheme, host, port
