@@ -1,16 +1,13 @@
 """Helper functions for clients in Jina."""
-__copyright__ = "Copyright (c) 2020 Jina AI Limited. All rights reserved."
-__license__ = "Apache-2.0"
 
 from functools import wraps
-from typing import Callable
+from typing import Callable, Optional
 
-from .. import Response
-from ..excepts import BadClientCallback
-from ..helper import colored
-from ..importer import ImportExtensions
-from ..logging import JinaLogger
-from ..proto import jina_pb2
+from jina.excepts import BadClientCallback
+from jina.helper import get_rich_console
+from jina.logging.logger import JinaLogger
+from jina.proto import jina_pb2
+from jina.types.request.data import Response
 
 
 def pprint_routes(resp: 'Response', stack_limit: int = 3):
@@ -18,42 +15,29 @@ def pprint_routes(resp: 'Response', stack_limit: int = 3):
 
     :param resp: the :class:`Response` object
     :param stack_limit: traceback limit
-    :return:
     """
-    from textwrap import fill
-
     routes = resp.routes
 
-    header = [colored(v, attrs=['bold']) for v in ('Pod', 'Time', 'Exception')]
+    from rich import box
+    from rich.table import Table
 
-    with ImportExtensions(required=False):
-        from prettytable import PrettyTable, ALL
-
-        table = PrettyTable(field_names=header, align='l', hrules=ALL)
-        add_row = table.add_row
-        visualize = print
+    table = Table(box=box.SIMPLE)
+    for v in ('Executor', 'Time', 'Exception'):
+        table.add_column(v)
 
     for route in routes:
         status_icon = '🟢'
         if route.status.code == jina_pb2.StatusProto.ERROR:
             status_icon = '🔴'
-        elif route.status.code == jina_pb2.StatusProto.ERROR_CHAINED:
-            status_icon = '⚪'
 
-        add_row(
-            [
-                f'{status_icon} {route.pod}',
-                f'{route.start_time.ToMilliseconds() - routes[0].start_time.ToMilliseconds()}ms',
-                fill(
-                    ''.join(route.status.exception.stacks[-stack_limit:]),
-                    width=50,
-                    break_long_words=False,
-                    replace_whitespace=False,
-                ),
-            ]
+        table.add_row(
+            f'{status_icon} {route.executor}',
+            f'{route.start_time.ToMilliseconds() - routes[0].start_time.ToMilliseconds()}ms',
+            ''.join(route.status.exception.stacks[-stack_limit:]),
         )
 
-    visualize(table)
+    console = get_rich_console()
+    console.print(table)
 
 
 def _safe_callback(func: Callable, continue_on_error: bool, logger) -> Callable:
@@ -88,9 +72,34 @@ def callback_exec(
     :param continue_on_error: whether to continue on error
     :param logger: a logger instance
     """
-    if on_error and response.status.code >= jina_pb2.StatusProto.ERROR:
-        _safe_callback(on_error, continue_on_error, logger)(response)
-    elif on_done:
+    if on_error and response.header.status.code >= jina_pb2.StatusProto.ERROR:
+
+        @wraps(on_error)
+        def on_error_wrap(resp):
+            on_error(resp, None)
+
+        _safe_callback(on_error_wrap, continue_on_error, logger)(response)
+    elif on_done and response.header.status.code == jina_pb2.StatusProto.SUCCESS:
         _safe_callback(on_done, continue_on_error, logger)(response)
     if on_always:
         _safe_callback(on_always, continue_on_error, logger)(response)
+
+
+def callback_exec_on_error(
+    on_error: Callable,
+    exception: Exception,
+    logger: JinaLogger,
+    response: Optional = None,
+) -> None:
+    """Execute the on_error callback with the response, Use when an error outside the response status was thrown.
+    :param on_error: the on_error callback
+    :param exception: the exception with was thrown and led to the call of on_error
+    :param logger: a logger instance
+    :param response: the response
+    """
+
+    @wraps(on_error)
+    def on_error_wrap(resp):
+        on_error(resp, exception)
+
+    _safe_callback(on_error_wrap, False, logger)(response)
