@@ -762,7 +762,7 @@ def warn_unknown_args(unknown_args: List[str]):
     :param unknown_args: arguments that are possibly unknown to Jina
     """
 
-    from cli.lookup import _build_lookup_table
+    from jina_cli.lookup import _build_lookup_table
 
     all_args = _build_lookup_table()[0]
     has_migration_tip = False
@@ -838,6 +838,8 @@ class ArgNamespace:
             args += positional_args
         p_args, unknown_args = parser.parse_known_args(args)
         unknown_args = list(filter(lambda x: x.startswith('--'), unknown_args))
+        if '--jcloud' in unknown_args:
+            unknown_args.remove('--jcloud')
         if warn_unknown and unknown_args:
             _leftovers = set(unknown_args)
             if fallback_parsers:
@@ -944,7 +946,6 @@ def get_full_version() -> Optional[Tuple[Dict, Dict]]:
             'jina': __version__,
             'docarray': __docarray_version__,
             'jina-proto': __proto_version__,
-            'jina-vcs-tag': os.environ.get('JINA_VCS_VERSION', __unset_msg__),
             'protobuf': google.protobuf.__version__,
             'proto-backend': api_implementation._default_implementation_type,
             'grpcio': getattr(grpc, '__version__', _grpcio_metadata.__version__),
@@ -993,7 +994,8 @@ def _update_policy():
         try:
             import uvloop
 
-            asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())
+            if not isinstance(asyncio.get_event_loop_policy(), uvloop.EventLoopPolicy):
+                asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())
         except ModuleNotFoundError:
             warnings.warn(
                 'Install `uvloop` via `pip install "jina[uvloop]"` for better performance.'
@@ -1006,13 +1008,13 @@ def get_or_reuse_loop():
 
     :return: A new eventloop or reuse the current opened eventloop.
     """
+    _update_policy()
     try:
-        loop = asyncio.get_running_loop()
+        loop = asyncio.get_event_loop()
         if loop.is_closed():
             raise RuntimeError
     except RuntimeError:
-        _update_policy()
-        # no running event loop
+        # no event loop
         # create a new loop
         loop = asyncio.new_event_loop()
         asyncio.set_event_loop(loop)
@@ -1269,23 +1271,19 @@ def iscoroutinefunction(func: Callable):
 def run_async(func, *args, **kwargs):
     """Generalized asyncio.run for jupyter notebook.
 
-    When running inside jupyter, an eventloop is already exist, can't be stopped, can't be killed.
+    When running inside jupyter, an eventloop already exists, can't be stopped, can't be killed.
     Directly calling asyncio.run will fail, as This function cannot be called when another asyncio event loop
     is running in the same thread.
 
     .. see_also:
         https://stackoverflow.com/questions/55409641/asyncio-run-cannot-be-called-from-a-running-event-loop
 
-    call `run_async(my_function, any_event_loop=True, *args, **kwargs)` to enable run with any eventloop
-
     :param func: function to run
     :param args: parameters
     :param kwargs: key-value parameters
     :return: asyncio.run(func)
     """
 
-    any_event_loop = kwargs.pop('any_event_loop', False)
-
     class _RunThread(threading.Thread):
         """Create a running thread when in Jupyter notebook."""
 
@@ -1301,7 +1299,7 @@ def run_async(func, *args, **kwargs):
     if loop and loop.is_running():
         # eventloop already exist
         # running inside Jupyter
-        if any_event_loop or is_jupyter():
+        if is_jupyter():
             thread = _RunThread()
             thread.start()
             thread.join()
@@ -1322,7 +1320,7 @@ def run_async(func, *args, **kwargs):
                 'please report this issue here: https://github.com/jina-ai/jina'
             )
     else:
-        return get_or_reuse_loop().run_until_complete(func(*args, **kwargs))
+        return asyncio.run(func(*args, **kwargs))
 
 
 def slugify(value):
@@ -1523,20 +1521,37 @@ def get_rich_console():
     :return: rich console
     """
     return Console(
-        force_terminal=True,
+        force_terminal=True if 'PYCHARM_HOSTED' in os.environ else None,
         color_system=None if 'JINA_LOG_NO_COLOR' in os.environ else 'auto',
-    )  # It forces render in any terminal, especially in PyCharm
+    )
 
 
 from jina.parsers import set_client_cli_parser
 
+__default_port_client__ = 80
+__default_port_tls_client__ = 443
 
-def parse_client(kwargs):
+
+def parse_client(kwargs) -> Namespace:
+    """
+    Parse the kwargs for the Client
+
+    :param kwargs: kwargs to be parsed
+
+    :return: parsed argument.
+    """
     kwargs = _parse_kwargs(kwargs)
-    return ArgNamespace.kwargs2namespace(
+    args = ArgNamespace.kwargs2namespace(
         kwargs, set_client_cli_parser(), warn_unknown=True
     )
 
+    if not args.port:
+        args.port = (
+            __default_port_client__ if not args.tls else __default_port_tls_client__
+        )
+
+    return args
+
 
 def _parse_kwargs(kwargs: Dict[str, Any]) -> Dict[str, Any]:
     if 'host' in kwargs.keys():
@@ -1557,18 +1572,11 @@ def _parse_kwargs(kwargs: Dict[str, Any]) -> Dict[str, Any]:
                 elif value:
                     kwargs[key] = value
 
-    kwargs = _add_default_port_tls(kwargs)
     kwargs = _delete_host_slash(kwargs)
 
     return kwargs
 
 
-def _add_default_port_tls(kwargs: Dict[str, Any]) -> Dict[str, Any]:
-    if ('tls' in kwargs) and ('port' not in kwargs):
-        kwargs['port'] = 443
-    return kwargs
-
-
 def _delete_host_slash(kwargs: Dict[str, Any]) -> Dict[str, Any]:
     if 'host' in kwargs:
         if kwargs['host'][-1] == '/':
