@@ -23,7 +23,7 @@ def mixin_client_gateway_parser(parser):
     gp.add_argument(
         '--port',
         type=int,
-        default=helper.random_port(),
+        default=None,
         help='The port of the Gateway, which the client should connect to.',
     )
 
@@ -81,10 +81,16 @@ def mixin_gateway_parser(parser):
 
     gp.add_argument(
         '--compression',
-        type=str,
-        default='NoCompression',
-        help='The compression mechanism used when sending requests to Executors. Possibilites are: `NoCompression, '
-        'Gzip, Deflate`. For more details, check https://grpc.github.io/grpc/python/grpc.html#compression.',
+        choices=['NoCompression', 'Deflate', 'Gzip'],
+        help='The compression mechanism used when sending requests from the Head to the WorkerRuntimes. For more details, '
+        'check https://grpc.github.io/grpc/python/grpc.html#compression.',
+    )
+
+    gp.add_argument(
+        '--timeout-send',
+        type=int,
+        default=None,
+        help='The timeout in milliseconds used when sending data requests to Executors, -1 means no timeout, disabled by default',
     )
 
 
@@ -152,18 +158,11 @@ def mixin_http_gateway_parser(parser=None):
         ''',
     )
 
-    gp.add_argument(
-        '--default-swagger-ui',
-        action='store_true',
-        default=False,
-        help='If set, the default swagger ui is used for `/docs` endpoint. ',
-    )
-
     gp.add_argument(
         '--no-debug-endpoints',
         action='store_true',
         default=False,
-        help='If set, /status /post endpoints are removed from HTTP interface. ',
+        help='If set, `/status` `/post` endpoints are removed from HTTP interface. ',
     )
 
     gp.add_argument(
@@ -171,7 +170,7 @@ def mixin_http_gateway_parser(parser=None):
         action='store_true',
         default=False,
         help='''
-        If set, /index, /search, /update, /delete endpoints are removed from HTTP interface.
+        If set, `/index`, `/search`, `/update`, `/delete` endpoints are removed from HTTP interface.
 
         Any executor that has `@requests(on=...)` bind with those values will receive data requests.
         ''',
