@@ -3,11 +3,12 @@ import contextlib
 import ipaddress
 import os
 from collections import defaultdict
-from typing import Dict, List, Optional, Tuple
+from typing import Dict, List, Optional, Set, Tuple
 from urllib.parse import urlparse
 
 import grpc
 from grpc.aio import AioRpcError
+from grpc_health.v1 import health_pb2, health_pb2_grpc
 from grpc_reflection.v1alpha.reflection_pb2 import ServerReflectionRequest
 from grpc_reflection.v1alpha.reflection_pb2_grpc import ServerReflectionStub
 
@@ -16,7 +17,6 @@ from jina.importer import ImportExtensions
 from jina.logging.logger import JinaLogger
 from jina.proto import jina_pb2, jina_pb2_grpc
 from jina.types.request import Request
-from jina.types.request.control import ControlRequest
 from jina.types.request.data import DataRequest
 
 TLS_PROTOCOL_SCHEMES = ['grpcs', 'https', 'wss']
@@ -27,6 +27,9 @@ if TYPE_CHECKING:
     from prometheus_client import CollectorRegistry
 
 
+DEFAULT_MINIMUM_RETRIES = 3
+
+
 class ReplicaList:
     """
     Maintains a list of connections to replicas and uses round robin for selecting a replica
@@ -146,6 +149,7 @@ class GrpcConnectionPool:
     K8S_PORT_USES_AFTER = 8082
     K8S_PORT_USES_BEFORE = 8081
     K8S_PORT = 8080
+    K8S_PORT_MONITORING = 9090
 
     class ConnectionStubs:
         """
@@ -153,11 +157,11 @@ class GrpcConnectionPool:
         """
 
         STUB_MAPPING = {
-            'jina.JinaControlRequestRPC': jina_pb2_grpc.JinaControlRequestRPCStub,
             'jina.JinaDataRequestRPC': jina_pb2_grpc.JinaDataRequestRPCStub,
             'jina.JinaSingleDataRequestRPC': jina_pb2_grpc.JinaSingleDataRequestRPCStub,
             'jina.JinaDiscoverEndpointsRPC': jina_pb2_grpc.JinaDiscoverEndpointsRPCStub,
             'jina.JinaRPC': jina_pb2_grpc.JinaRPCStub,
+            'jina.JinaInfoRPC': jina_pb2_grpc.JinaInfoRPCStub,
         }
 
         def __init__(self, address, channel, summary):
@@ -175,7 +179,6 @@ class GrpcConnectionPool:
             stubs = defaultdict(lambda: None)
             for service in available_services:
                 stubs[service] = self.STUB_MAPPING[service](self.channel)
-            self.control_stub = stubs['jina.JinaControlRequestRPC']
             self.data_list_stub = stubs['jina.JinaDataRequestRPC']
             self.single_data_stub = stubs['jina.JinaSingleDataRequestRPC']
             self.stream_stub = stubs['jina.JinaRPC']
@@ -265,20 +268,6 @@ class GrpcConnectionPool:
                     raise ValueError(
                         'Can not send list of DataRequests. gRPC endpoint not available.'
                     )
-            elif request_type == ControlRequest:
-                if self.control_stub:
-                    call_result = self.control_stub.process_control(
-                        requests[0], timeout=timeout
-                    )
-                    metadata, response = (
-                        await call_result.trailing_metadata(),
-                        await call_result,
-                    )
-                    return response, metadata
-                else:
-                    raise ValueError(
-                        'Can not send ControlRequest. gRPC endpoint not available.'
-                    )
             else:
                 raise ValueError(f'Unsupported request type {type(requests[0])}')
 
@@ -310,6 +299,7 @@ class GrpcConnectionPool:
             entity_id: Optional[int] = None,
             increase_access_count: bool = True,
         ) -> ReplicaList:
+            # returns all replicas of a given deployment, using a given shard
             if deployment in self._deployments:
                 type_ = 'heads' if head else 'shards'
                 if entity_id is None and head:
@@ -324,6 +314,8 @@ class GrpcConnectionPool:
                 return None
 
         def get_replicas_all_shards(self, deployment: str) -> List[ReplicaList]:
+            # returns all replicas of a given deployment, for all available shards
+            # result is a list of 'shape' (num_shards, num_replicas), containing all replicas for all shards
             replicas = []
             if deployment in self._deployments:
                 for shard_id in self._deployments[deployment]['shards']:
@@ -434,25 +426,15 @@ class GrpcConnectionPool:
     def __init__(
         self,
         logger: Optional[JinaLogger] = None,
-        compression: str = 'NoCompression',
+        compression: Optional[str] = None,
         metrics_registry: Optional['CollectorRegistry'] = None,
     ):
         self._logger = logger or JinaLogger(self.__class__.__name__)
-        GRPC_COMPRESSION_MAP = {
-            'NoCompression'.lower(): grpc.Compression.NoCompression,
-            'Gzip'.lower(): grpc.Compression.Gzip,
-            'Deflate'.lower(): grpc.Compression.Deflate,
-        }
-        if compression.lower() not in GRPC_COMPRESSION_MAP:
-            import warnings
 
-            warnings.warn(
-                message=f'Your compression "{compression}" is not supported. Supported '
-                f'algorithms are `Gzip`, `Deflate` and `NoCompression`. NoCompression will be used as '
-                f'default'
-            )
-        self.compression = GRPC_COMPRESSION_MAP.get(
-            compression.lower(), grpc.Compression.NoCompression
+        self.compression = (
+            getattr(grpc.Compression, compression)
+            if compression
+            else grpc.Compression.NoCompression
         )
 
         if metrics_registry:
@@ -482,6 +464,7 @@ class GrpcConnectionPool:
         polling_type: PollingType = PollingType.ANY,
         endpoint: Optional[str] = None,
         timeout: Optional[float] = None,
+        retries: Optional[int] = -1,
     ) -> List[asyncio.Task]:
         """Send a single message to target via one or all of the pooled connections, depending on polling_type. Convenience function wrapper around send_request.
         :param request: a single request to send
@@ -491,6 +474,7 @@ class GrpcConnectionPool:
         :param polling_type: defines if the message should be send to any or all pooled connections for the target
         :param endpoint: endpoint to target with the request
         :param timeout: timeout for sending the requests
+        :param retries: number of retries per gRPC call. If <0 it defaults to max(3, num_replicas)
         :return: list of asyncio.Task items for each send call
         """
         return self.send_requests(
@@ -501,6 +485,7 @@ class GrpcConnectionPool:
             polling_type=polling_type,
             endpoint=endpoint,
             timeout=timeout,
+            retries=retries,
         )
 
     def send_requests(
@@ -512,33 +497,37 @@ class GrpcConnectionPool:
         polling_type: PollingType = PollingType.ANY,
         endpoint: Optional[str] = None,
         timeout: Optional[float] = None,
+        retries: Optional[int] = -1,
     ) -> List[asyncio.Task]:
         """Send a request to target via one or all of the pooled connections, depending on polling_type
 
-        :param requests: request (DataRequest/ControlRequest) to send
+        :param requests: request (DataRequest) to send
         :param deployment: name of the Jina deployment to send the request to
         :param head: If True it is send to the head, otherwise to the worker pods
         :param shard_id: Send to a specific shard of the deployment, ignored for polling ALL
         :param polling_type: defines if the request should be send to any or all pooled connections for the target
         :param endpoint: endpoint to target with the requests
         :param timeout: timeout for sending the requests
+        :param retries: number of retries per gRPC call. If <0 it defaults to max(3, num_replicas)
         :return: list of asyncio.Task items for each send call
         """
         results = []
         connections = []
         if polling_type == PollingType.ANY:
-            connection_list = self._connections.get_replicas(deployment, head, shard_id)
-            if connection_list:
-                connections.append(connection_list.get_next_connection())
+            replica_list = self._connections.get_replicas(deployment, head, shard_id)
+            if replica_list:
+                connections.append(replica_list)
         elif polling_type == PollingType.ALL:
-            connection_lists = self._connections.get_replicas_all_shards(deployment)
-            for connection_list in connection_lists:
-                connections.append(connection_list.get_next_connection())
+            shard_replica_lists = self._connections.get_replicas_all_shards(deployment)
+            for replica_list in shard_replica_lists:
+                connections.append(replica_list)
         else:
             raise ValueError(f'Unsupported polling type {polling_type}')
 
-        for connection in connections:
-            task = self._send_requests(requests, connection, endpoint, timeout=timeout)
+        for replica_list in connections:
+            task = self._send_requests(
+                requests, replica_list, endpoint, timeout=timeout, retries=retries
+            )
             results.append(task)
 
         return results
@@ -573,6 +562,7 @@ class GrpcConnectionPool:
         head: bool = False,
         shard_id: Optional[int] = None,
         timeout: Optional[float] = None,
+        retries: Optional[int] = -1,
     ) -> asyncio.Task:
         """Send msg to target via only one of the pooled connections
         :param request: request to send
@@ -580,6 +570,7 @@ class GrpcConnectionPool:
         :param head: If True it is send to the head, otherwise to the worker pods
         :param shard_id: Send to a specific shard of the deployment, ignored for polling ALL
         :param timeout: timeout for sending the requests
+        :param retries: number of retries per gRPC call. If <0 it defaults to max(3, num_replicas)
         :return: asyncio.Task representing the send call
         """
         return self.send_requests_once(
@@ -588,6 +579,7 @@ class GrpcConnectionPool:
             head=head,
             shard_id=shard_id,
             timeout=timeout,
+            retries=retries,
         )
 
     def send_requests_once(
@@ -598,6 +590,7 @@ class GrpcConnectionPool:
         shard_id: Optional[int] = None,
         endpoint: Optional[str] = None,
         timeout: Optional[float] = None,
+        retries: Optional[int] = -1,
     ) -> asyncio.Task:
         """Send a request to target via only one of the pooled connections
 
@@ -607,12 +600,14 @@ class GrpcConnectionPool:
         :param shard_id: Send to a specific shard of the deployment, ignored for polling ALL
         :param endpoint: endpoint to target with the requests
         :param timeout: timeout for sending the requests
+        :param retries: number of retries per gRPC call. If <0 it defaults to max(3, num_replicas)
         :return: asyncio.Task representing the send call
         """
         replicas = self._connections.get_replicas(deployment, head, shard_id)
         if replicas:
-            connection = replicas.get_next_connection()
-            return self._send_requests(requests, connection, endpoint, timeout=timeout)
+            return self._send_requests(
+                requests, replicas, endpoint, timeout=timeout, retries=retries
+            )
         else:
             self._logger.debug(
                 f'no available connections for deployment {deployment} and shard {shard_id}'
@@ -682,7 +677,8 @@ class GrpcConnectionPool:
         e: AioRpcError,
         retry_i: int = 0,
         request_id: str = '',
-        dest_addr: str = '',
+        dest_addr: Set[str] = {''},
+        num_retries: int = 0,
     ):
         # connection failures and cancelled requests should be retried
         # all other cases should not be retried and will be raised immediately
@@ -693,9 +689,13 @@ class GrpcConnectionPool:
         if (
             e.code() != grpc.StatusCode.UNAVAILABLE
             and e.code() != grpc.StatusCode.CANCELLED
+            and e.code() != grpc.StatusCode.DEADLINE_EXCEEDED
         ):
             raise
-        elif e.code() == grpc.StatusCode.UNAVAILABLE and retry_i >= 2:
+        elif (
+            e.code() == grpc.StatusCode.UNAVAILABLE
+            or e.code() == grpc.StatusCode.DEADLINE_EXCEEDED
+        ) and retry_i >= num_retries - 1:
             self._logger.debug(f'GRPC call failed, retries exhausted')
             from jina.excepts import InternalNetworkError
 
@@ -707,23 +707,35 @@ class GrpcConnectionPool:
             )
         else:
             self._logger.debug(
-                f'GRPC call failed with code {e.code()}, retry attempt {retry_i + 1}/3'
+                f'GRPC call failed with code {e.code()}, retry attempt {retry_i + 1}/{num_retries - 1}.'
+                f' Trying next replica, if available.'
             )
 
     def _send_requests(
         self,
         requests: List[Request],
-        connection: ConnectionStubs,
+        connections: ReplicaList,
         endpoint: Optional[str] = None,
         timeout: Optional[float] = None,
+        retries: Optional[int] = -1,
     ) -> asyncio.Task:
         # this wraps the awaitable object from grpc as a coroutine so it can be used as a task
         # the grpc call function is not a coroutine but some _AioCall
         async def task_wrapper():
             metadata = (('endpoint', endpoint),) if endpoint else None
-            for i in range(3):
+            tried_addresses = set()
+            if retries is None or retries < 0:
+                total_num_tries = (
+                    max(DEFAULT_MINIMUM_RETRIES, len(connections.get_all_connections()))
+                    + 1
+                )
+            else:
+                total_num_tries = 1 + retries  # try once, then do all the retries
+            for i in range(total_num_tries):
+                current_connection = connections.get_next_connection()
+                tried_addresses.add(current_connection.address)
                 try:
-                    return await connection.send_requests(
+                    return await current_connection.send_requests(
                         requests=requests,
                         metadata=metadata,
                         compression=self.compression,
@@ -734,7 +746,8 @@ class GrpcConnectionPool:
                         e=e,
                         retry_i=i,
                         request_id=requests[0].request_id,
-                        dest_addr=connection.address,
+                        dest_addr=tried_addresses,
+                        num_retries=total_num_tries,
                     )
 
         return asyncio.create_task(task_wrapper())
@@ -805,81 +818,6 @@ class GrpcConnectionPool:
 
         return insecure_channel(address, options)
 
-    @staticmethod
-    def activate_worker_sync(
-        worker_host: str,
-        worker_port: int,
-        target_head: str,
-        shard_id: Optional[int] = None,
-    ) -> ControlRequest:
-        """
-        Register a given worker to a head by sending an activate request
-
-        :param worker_host: the host address of the worker
-        :param worker_port: the port of the worker
-        :param target_head: address of the head to send the activate request to
-        :param shard_id: id of the shard the worker belongs to
-        :returns: the response request
-        """
-        activate_request = ControlRequest(command='ACTIVATE')
-        activate_request.add_related_entity(
-            'worker', worker_host, worker_port, shard_id
-        )
-
-        if os.name != 'nt':
-            os.unsetenv('http_proxy')
-            os.unsetenv('https_proxy')
-
-        return GrpcConnectionPool.send_request_sync(activate_request, target_head)
-
-    @staticmethod
-    async def activate_worker(
-        worker_host: str,
-        worker_port: int,
-        target_head: str,
-        shard_id: Optional[int] = None,
-    ) -> ControlRequest:
-        """
-        Register a given worker to a head by sending an activate request
-
-        :param worker_host: the host address of the worker
-        :param worker_port: the port of the worker
-        :param target_head: address of the head to send the activate request to
-        :param shard_id: id of the shard the worker belongs to
-        :returns: the response request
-        """
-        activate_request = ControlRequest(command='ACTIVATE')
-        activate_request.add_related_entity(
-            'worker', worker_host, worker_port, shard_id
-        )
-        return await GrpcConnectionPool.send_request_async(
-            activate_request, target_head
-        )
-
-    @staticmethod
-    async def deactivate_worker(
-        worker_host: str,
-        worker_port: int,
-        target_head: str,
-        shard_id: Optional[int] = None,
-    ) -> ControlRequest:
-        """
-        Remove a given worker to a head by sending a deactivate request
-
-        :param worker_host: the host address of the worker
-        :param worker_port: the port of the worker
-        :param target_head: address of the head to send the deactivate request to
-        :param shard_id: id of the shard the worker belongs to
-        :returns: the response request
-        """
-        activate_request = ControlRequest(command='DEACTIVATE')
-        activate_request.add_related_entity(
-            'worker', worker_host, worker_port, shard_id
-        )
-        return await GrpcConnectionPool.send_request_async(
-            activate_request, target_head
-        )
-
     @staticmethod
     def send_request_sync(
         request: Request,
@@ -909,22 +847,51 @@ class GrpcConnectionPool:
                     tls=tls,
                     root_certificates=root_certificates,
                 ) as channel:
-                    if type(request) == DataRequest:
-                        metadata = (('endpoint', endpoint),) if endpoint else None
-                        stub = jina_pb2_grpc.JinaSingleDataRequestRPCStub(channel)
-                        response, call = stub.process_single_data.with_call(
-                            request,
-                            timeout=timeout,
-                            metadata=metadata,
-                        )
-                    elif type(request) == ControlRequest:
-                        stub = jina_pb2_grpc.JinaControlRequestRPCStub(channel)
-                        response = stub.process_control(request, timeout=timeout)
+                    metadata = (('endpoint', endpoint),) if endpoint else None
+                    stub = jina_pb2_grpc.JinaSingleDataRequestRPCStub(channel)
+                    response, call = stub.process_single_data.with_call(
+                        request,
+                        timeout=timeout,
+                        metadata=metadata,
+                    )
                     return response
             except grpc.RpcError as e:
                 if e.code() != grpc.StatusCode.UNAVAILABLE or i == 2:
                     raise
 
+    @staticmethod
+    def send_health_check_sync(
+        target: str,
+        timeout=100.0,
+        tls=False,
+        root_certificates: Optional[str] = None,
+    ) -> health_pb2.HealthCheckResponse:
+        """
+        Sends a request synchronously to the target via grpc
+
+        :param target: where to send the request to, like 127.0.0.1:8080
+        :param timeout: timeout for the send
+        :param tls: if True, use tls encryption for the grpc channel
+        :param root_certificates: the path to the root certificates for tls, only used if tls is True
+
+        :returns: the response health check
+        """
+
+        for i in range(3):
+            try:
+                with GrpcConnectionPool.get_grpc_channel(
+                    target,
+                    tls=tls,
+                    root_certificates=root_certificates,
+                ) as channel:
+                    health_check_req = health_pb2.HealthCheckRequest()
+                    health_check_req.service = ''
+                    stub = health_pb2_grpc.HealthStub(channel)
+                    return stub.Check(health_check_req, timeout=timeout)
+            except grpc.RpcError as e:
+                if e.code() != grpc.StatusCode.UNAVAILABLE or i == 2:
+                    raise
+
     @staticmethod
     def send_requests_sync(
         requests: List[Request],
@@ -1004,12 +971,8 @@ class GrpcConnectionPool:
             tls=tls,
             root_certificates=root_certificates,
         ) as channel:
-            if type(request) == DataRequest:
-                stub = jina_pb2_grpc.JinaSingleDataRequestRPCStub(channel)
-                return await stub.process_single_data(request, timeout=timeout)
-            elif type(request) == ControlRequest:
-                stub = jina_pb2_grpc.JinaControlRequestRPCStub(channel)
-                return await stub.process_control(request, timeout=timeout)
+            stub = jina_pb2_grpc.JinaSingleDataRequestRPCStub(channel)
+            return await stub.process_single_data(request, timeout=timeout)
 
     @staticmethod
     def create_async_channel_stub(
@@ -1023,7 +986,7 @@ class GrpcConnectionPool:
         :param root_certificates: the path to the root certificates for tls, only u
         :param summary: Optional Prometheus summary object
 
-        :returns: DataRequest/ControlRequest stubs and an async grpc channel
+        :returns: DataRequest stubs and an async grpc channel
         """
         channel = GrpcConnectionPool.get_grpc_channel(
             address,
@@ -1056,7 +1019,11 @@ class GrpcConnectionPool:
                 [
                     service.name
                     for service in res.list_services_response.service
-                    if service.name != 'grpc.reflection.v1alpha.ServerReflection'
+                    if service.name
+                    not in {
+                        'grpc.reflection.v1alpha.ServerReflection',
+                        'jina.JinaGatewayDryRunRPC',
+                    }
                 ]
             )
         return service_names[0]
