@@ -1,8 +1,8 @@
-import os
 import asyncio
+import logging
+import os
 
 from jina import __default_host__
-
 from jina.importer import ImportExtensions
 from jina.serve.runtimes.gateway import GatewayRuntime
 from jina.serve.runtimes.gateway.http.app import get_fastapi_app
@@ -48,9 +48,25 @@ class HTTPGatewayRuntime(GatewayRuntime):
                 """
                 await self.main_loop()
 
+        if 'JINA_DISABLE_HEALTHCHECK_LOGS' in os.environ:
+
+            class _EndpointFilter(logging.Filter):
+                def filter(self, record: logging.LogRecord) -> bool:
+                    # NOTE: space is important after `GET /`, else all logs will be disabled.
+                    return record.getMessage().find("GET / ") == -1
+
+            # Filter out healthcheck endpoint `GET /`
+            logging.getLogger("uvicorn.access").addFilter(_EndpointFilter())
+
         from jina.helper import extend_rest_interface
 
         uvicorn_kwargs = self.args.uvicorn_kwargs or {}
+
+        for ssl_file in ['ssl_keyfile', 'ssl_certfile']:
+            if getattr(self.args, ssl_file):
+                if ssl_file not in uvicorn_kwargs.keys():
+                    uvicorn_kwargs[ssl_file] = getattr(self.args, ssl_file)
+
         self._set_topology_graph()
         self._set_connection_pool()
         self._server = UviServer(
@@ -61,10 +77,11 @@ class HTTPGatewayRuntime(GatewayRuntime):
                         topology_graph=self._topology_graph,
                         connection_pool=self._connection_pool,
                         logger=self.logger,
+                        metrics_registry=self.metrics_registry,
                     )
                 ),
                 host=__default_host__,
-                port=self.args.port_expose,
+                port=self.args.port,
                 log_level=os.getenv('JINA_LOG_LEVEL', 'error').lower(),
                 **uvicorn_kwargs
             )
