@@ -2,8 +2,6 @@ import argparse
 import json
 from typing import TYPE_CHECKING, Dict, List, Optional
 
-from docarray import Document
-
 from jina import __version__
 from jina.clients.request import request_generator
 from jina.enums import DataInputType
@@ -11,7 +9,6 @@ from jina.excepts import InternalNetworkError
 from jina.helper import get_full_version
 from jina.importer import ImportExtensions
 from jina.logging.logger import JinaLogger
-from jina.logging.profile import used_memory_readable
 
 if TYPE_CHECKING:
     from prometheus_client import CollectorRegistry
@@ -40,24 +37,19 @@ def get_fastapi_app(
     with ImportExtensions(required=True):
         from fastapi import FastAPI, Response, status
         from fastapi.middleware.cors import CORSMiddleware
-        from fastapi.responses import HTMLResponse
-        from starlette.requests import Request
 
         from jina.serve.runtimes.gateway.http.models import (
             JinaEndpointRequestModel,
             JinaRequestModel,
             JinaResponseModel,
-            JinaStatusModel,
         )
 
-    docs_url = '/docs'
     app = FastAPI(
         title=args.title or 'My Jina Service',
         description=args.description
         or 'This is my awesome service. You can set `title` and `description` in your `Flow` or `Gateway` '
-        'to customize this text.',
+        'to customize the title and description.',
         version=__version__,
-        docs_url=docs_url if args.default_swagger_ui else None,
     )
 
     if args.cors:
@@ -68,9 +60,7 @@ def get_fastapi_app(
             allow_methods=['*'],
             allow_headers=['*'],
         )
-        logger.warning(
-            'CORS is enabled. This service is now accessible from any website!'
-        )
+        logger.warning('CORS is enabled. This service is accessible from any website!')
 
     from jina.serve.runtimes.gateway.request_handling import RequestHandler
     from jina.serve.stream import RequestStreamer
@@ -104,21 +94,62 @@ def get_fastapi_app(
 
         @app.get(
             path='/',
-            summary='Get the health of Jina service',
+            summary='Get the health of Jina Gateway service',
             response_model=JinaHealthModel,
         )
-        async def _health():
+        async def _gateway_health():
             """
-            Get the health of this Jina service.
+            Get the health of this Gateway service.
             .. # noqa: DAR201
 
             """
             return {}
 
+        from docarray import DocumentArray
+
+        from jina.proto import jina_pb2
+        from jina.serve.executors import __dry_run_endpoint__
+        from jina.serve.runtimes.gateway.http.models import (
+            PROTO_TO_PYDANTIC_MODELS,
+            JinaInfoModel,
+        )
+        from jina.types.request.status import StatusMessage
+
+        @app.get(
+            path='/dry_run',
+            summary='Get the readiness of Jina Flow service, sends an empty DocumentArray to the complete Flow to '
+            'validate connectivity',
+            response_model=PROTO_TO_PYDANTIC_MODELS.StatusProto,
+        )
+        async def _flow_health():
+            """
+            Get the health of the complete Flow service.
+            .. # noqa: DAR201
+
+            """
+
+            da = DocumentArray()
+
+            try:
+                _ = await _get_singleton_result(
+                    request_generator(
+                        exec_endpoint=__dry_run_endpoint__,
+                        data=da,
+                        data_type=DataInputType.DOCUMENT,
+                    )
+                )
+                status_message = StatusMessage()
+                status_message.set_code(jina_pb2.StatusProto.SUCCESS)
+                return status_message.to_dict()
+            except Exception as ex:
+                status_message = StatusMessage()
+                status_message.set_exception(ex)
+                return status_message.to_dict(use_integers_for_enums=True)
+
         @app.get(
             path='/status',
             summary='Get the status of Jina service',
-            response_model=JinaStatusModel,
+            response_model=JinaInfoModel,
             tags=['Debug'],
         )
         async def _status():
@@ -129,12 +160,12 @@ def get_fastapi_app(
 
             .. # noqa: DAR201
             """
-            _info = get_full_version()
-            return {
-                'jina': _info[0],
-                'envs': _info[1],
-                'used_memory': used_memory_readable(),
-            }
+            version, env_info = get_full_version()
+            for k, v in version.items():
+                version[k] = str(v)
+            for k, v in env_info.items():
+                env_info[k] = str(v)
+            return {'jina': version, 'envs': env_info}
 
         @app.post(
             path='/post',
@@ -175,7 +206,14 @@ def get_fastapi_app(
                     request_generator(**req_generator_input)
                 )
             except InternalNetworkError as err:
-                response.status_code = status.HTTP_503_SERVICE_UNAVAILABLE
+                import grpc
+
+                if err.code() == grpc.StatusCode.UNAVAILABLE:
+                    response.status_code = status.HTTP_503_SERVICE_UNAVAILABLE
+                elif err.code() == grpc.StatusCode.DEADLINE_EXCEEDED:
+                    response.status_code = status.HTTP_504_GATEWAY_TIMEOUT
+                else:
+                    response.status_code = status.HTTP_500_INTERNAL_SERVER_ERROR
                 result = bd  # send back the request
                 result['header'] = _generate_exception_header(
                     err
@@ -188,6 +226,8 @@ def get_fastapi_app(
     def _generate_exception_header(error: InternalNetworkError):
         import traceback
 
+        from jina.proto.serializer import DataRequest
+
         exception_dict = {
             'name': str(error.__class__),
             'stacks': [
@@ -196,7 +236,7 @@ def get_fastapi_app(
             'executor': '',
         }
         status_dict = {
-            'code': 3,  # status error
+            'code': DataRequest().status.ERROR,
             'description': error.details() if error.details() else '',
             'exception': exception_dict,
         }
@@ -266,20 +306,6 @@ def get_fastapi_app(
         for k, v in endpoints.items():
             expose_executor_endpoint(exec_endpoint=k, **v)
 
-    if not args.default_swagger_ui:
-
-        async def _render_custom_swagger_html(req: Request) -> HTMLResponse:
-            import urllib.request
-
-            swagger_url = 'https://api.jina.ai/swagger'
-            req = urllib.request.Request(
-                swagger_url, headers={'User-Agent': 'Mozilla/5.0'}
-            )
-            with urllib.request.urlopen(req) as f:
-                return HTMLResponse(f.read().decode())
-
-        app.add_route(docs_url, _render_custom_swagger_html, include_in_schema=False)
-
     if args.expose_graphql_endpoint:
         with ImportExtensions(required=True):
             from dataclasses import asdict
