@@ -4,8 +4,8 @@ import time
 from typing import TYPE_CHECKING, Callable, List, Optional
 
 import grpc.aio
-from docarray import DocumentArray
 
+from docarray import DocumentArray
 from jina.excepts import InternalNetworkError
 from jina.importer import ImportExtensions
 from jina.serve.networking import GrpcConnectionPool
@@ -30,7 +30,7 @@ class RequestHandler:
         metrics_registry: Optional['CollectorRegistry'] = None,
         runtime_name: Optional[str] = None,
     ):
-        self.request_init_time = {} if metrics_registry else None
+        self._request_init_time = {} if metrics_registry else None
         self._executor_endpoint_mapping = None
 
         if metrics_registry:
@@ -38,9 +38,9 @@ class RequestHandler:
                 required=True,
                 help_text='You need to install the `prometheus_client` to use the montitoring functionality of jina',
             ):
-                from prometheus_client import Summary
+                from prometheus_client import Gauge, Summary
 
-            self._summary = Summary(
+            self._receiving_request_metrics = Summary(
                 'receiving_request_seconds',
                 'Time spent processing request',
                 registry=metrics_registry,
@@ -48,8 +48,17 @@ class RequestHandler:
                 labelnames=('runtime_name',),
             ).labels(runtime_name)
 
+            self._pending_requests_metrics = Gauge(
+                'number_of_pending_requests',
+                'Number of pending requests',
+                registry=metrics_registry,
+                namespace='jina',
+                labelnames=('runtime_name',),
+            ).labels(runtime_name)
+
         else:
-            self._summary = None
+            self._receiving_request_metrics = None
+            self._pending_requests_metrics = None
 
     def handle_request(
         self, graph: 'TopologyGraph', connection_pool: 'GrpcConnectionPool'
@@ -63,22 +72,7 @@ class RequestHandler:
         """
 
         async def gather_endpoints(request_graph):
-            def _get_all_nodes(node, accum, accum_names):
-                if node.name not in accum_names:
-                    accum.append(node)
-                    accum_names.append(node.name)
-                for n in node.outgoing_nodes:
-                    _get_all_nodes(n, accum, accum_names)
-                return accum, accum_names
-
-            nodes = []
-            node_names = []
-            for origin_node in request_graph.origin_nodes:
-                subtree_nodes, subtree_node_names = _get_all_nodes(origin_node, [], [])
-                for st_node, st_node_name in zip(subtree_nodes, subtree_node_names):
-                    if st_node_name not in node_names:
-                        nodes.append(st_node)
-                        node_names.append(st_node_name)
+            nodes = request_graph.all_nodes
             try:
                 tasks_to_get_endpoints = [
                     node.get_endpoints(connection_pool) for node in nodes
@@ -89,7 +83,7 @@ class RequestHandler:
                 if err_code == grpc.StatusCode.UNAVAILABLE:
                     err._details = (
                         err.details()
-                        + f' |Gateway: Communication error with deployment at address {err.dest_addr}. Head or worker may be down.'
+                        + f' |Gateway: Communication error with deployment at address(es) {err.dest_addr}. Head or worker(s) may be down.'
                     )
                     raise err
                 else:
@@ -100,8 +94,10 @@ class RequestHandler:
                 self._executor_endpoint_mapping[node.name] = endp.endpoints
 
         def _handle_request(request: 'Request') -> 'asyncio.Future':
-            if self._summary:
-                self.request_init_time[request.request_id] = time.time()
+            if self._receiving_request_metrics:
+                self._request_init_time[request.request_id] = time.time()
+            if self._pending_requests_metrics:
+                self._pending_requests_metrics.inc()
             # important that the gateway needs to have an instance of the graph per request
             request_graph = copy.deepcopy(graph)
 
@@ -194,10 +190,14 @@ class RequestHandler:
                 if route.executor == 'gateway':
                     route.end_time.GetCurrentTime()
 
-            if self._summary:
-                self._summary.observe(
-                    time.time() - self.request_init_time[result.request_id]
-                )
+            if self._receiving_request_metrics:
+                init_time = self._request_init_time.pop(
+                    result.request_id
+                )  # need to pop otherwise it stays in memory forever
+                self._receiving_request_metrics.observe(time.time() - init_time)
+
+            if self._pending_requests_metrics:
+                self._pending_requests_metrics.dec()
 
             return result
 
