@@ -4,13 +4,14 @@ from abc import ABC
 from typing import List
 
 import grpc
+from grpc_health.v1 import health, health_pb2, health_pb2_grpc
 from grpc_reflection.v1alpha import reflection
 
+from jina.helper import get_full_version
 from jina.importer import ImportExtensions
 from jina.proto import jina_pb2, jina_pb2_grpc
 from jina.serve.runtimes.asyncio import AsyncNewLoopRuntime
 from jina.serve.runtimes.request_handlers.data_request_handler import DataRequestHandler
-from jina.types.request.control import ControlRequest
 from jina.types.request.data import DataRequest
 
 
@@ -26,14 +27,13 @@ class WorkerRuntime(AsyncNewLoopRuntime, ABC):
         :param args: args from CLI
         :param kwargs: keyword args
         """
+        self._health_servicer = health.HealthServicer(experimental_non_blocking=True)
         super().__init__(args, **kwargs)
 
     async def async_setup(self):
         """
         Start the DataRequestHandler and wait for the GRPC and Monitoring servers to start
         """
-        await self._async_setup_grpc_server()
-
         if self.metrics_registry:
             with ImportExtensions(
                 required=True,
@@ -41,15 +41,22 @@ class WorkerRuntime(AsyncNewLoopRuntime, ABC):
             ):
                 from prometheus_client import Summary
 
-            self._summary_time = Summary(
-                'receiving_request_seconds',
-                'Time spent processing request',
-                registry=self.metrics_registry,
-                namespace='jina',
-            ).time()
+            self._summary_time = (
+                Summary(
+                    'receiving_request_seconds',
+                    'Time spent processing request',
+                    registry=self.metrics_registry,
+                    namespace='jina',
+                    labelnames=('runtime_name',),
+                )
+                .labels(self.args.name)
+                .time()
+            )
         else:
             self._summary_time = contextlib.nullcontext()
 
+        await self._async_setup_grpc_server()
+
     async def _async_setup_grpc_server(self):
         """
         Start the DataRequestHandler and wait for the GRPC server to start
@@ -73,15 +80,25 @@ class WorkerRuntime(AsyncNewLoopRuntime, ABC):
             self, self._grpc_server
         )
         jina_pb2_grpc.add_JinaDataRequestRPCServicer_to_server(self, self._grpc_server)
-        jina_pb2_grpc.add_JinaControlRequestRPCServicer_to_server(
+
+        jina_pb2_grpc.add_JinaDiscoverEndpointsRPCServicer_to_server(
             self, self._grpc_server
         )
+        jina_pb2_grpc.add_JinaInfoRPCServicer_to_server(self, self._grpc_server)
         service_names = (
             jina_pb2.DESCRIPTOR.services_by_name['JinaSingleDataRequestRPC'].full_name,
             jina_pb2.DESCRIPTOR.services_by_name['JinaDataRequestRPC'].full_name,
-            jina_pb2.DESCRIPTOR.services_by_name['JinaControlRequestRPC'].full_name,
+            jina_pb2.DESCRIPTOR.services_by_name['JinaDiscoverEndpointsRPC'].full_name,
+            jina_pb2.DESCRIPTOR.services_by_name['JinaInfoRPC'].full_name,
             reflection.SERVICE_NAME,
         )
+        # Mark all services as healthy.
+        health_pb2_grpc.add_HealthServicer_to_server(
+            self._health_servicer, self._grpc_server
+        )
+
+        for service in service_names:
+            self._health_servicer.set(service, health_pb2.HealthCheckResponse.SERVING)
         reflection.enable_server_reflection(service_names, self._grpc_server)
         bind_addr = f'0.0.0.0:{self.args.port}'
         self.logger.debug(f'start listening on {bind_addr}')
@@ -103,6 +120,7 @@ class WorkerRuntime(AsyncNewLoopRuntime, ABC):
 
     async def async_teardown(self):
         """Close the data request handler"""
+        self._health_servicer.enter_graceful_shutdown()
         await self.async_cancel()
         self._data_request_handler.close()
 
@@ -116,6 +134,20 @@ class WorkerRuntime(AsyncNewLoopRuntime, ABC):
         """
         return await self.process_data([request], context)
 
+    async def endpoint_discovery(self, empty, context) -> jina_pb2.EndpointsProto:
+        """
+        Process the the call requested and return the list of Endpoints exposed by the Executor wrapped inside this Runtime
+
+        :param empty: The service expects an empty protobuf message
+        :param context: grpc context
+        :returns: the response request
+        """
+        endpointsProto = jina_pb2.EndpointsProto()
+        endpointsProto.endpoints.extend(
+            list(self._data_request_handler._executor.requests.keys())
+        )
+        return endpointsProto
+
     async def process_data(self, requests: List[DataRequest], context) -> DataRequest:
         """
         Process the received requests and return the result as a new request
@@ -144,31 +176,18 @@ class WorkerRuntime(AsyncNewLoopRuntime, ABC):
                 context.set_trailing_metadata((('is-error', 'true'),))
                 return requests[0]
 
-    async def process_control(self, request: ControlRequest, *args) -> ControlRequest:
+    async def _status(self, empty, context) -> jina_pb2.JinaInfoProto:
         """
-        Process the received control request and return the same request
+        Process the the call requested and return the JinaInfo of the Runtime
 
-        :param request: the control request to process
-        :param args: additional arguments in the grpc call, ignored
-        :returns: the input request
+        :param empty: The service expects an empty protobuf message
+        :param context: grpc context
+        :returns: the response request
         """
-        try:
-            if self.logger.debug_enabled:
-                self._log_control_request(request)
-
-            if request.command == 'STATUS':
-                pass
-            else:
-                raise RuntimeError(
-                    f'WorkerRuntime received unsupported ControlRequest command {request.command}'
-                )
-        except (RuntimeError, Exception) as ex:
-            self.logger.error(
-                f'{ex!r}' + f'\n add "--quiet-error" to suppress the exception details'
-                if not self.args.quiet_error
-                else '',
-                exc_info=not self.args.quiet_error,
-            )
-
-            request.add_exception(ex, self._data_request_handler._executor)
-        return request
+        infoProto = jina_pb2.JinaInfoProto()
+        version, env_info = get_full_version()
+        for k, v in version.items():
+            infoProto.jina[k] = str(v)
+        for k, v in env_info.items():
+            infoProto.envs[k] = str(v)
+        return infoProto
